@@ -16,12 +16,12 @@
 int todo_show_due_date = 0;
 int todo_show_priority = 0;
 int todo_show_time_remaining = 0;
+int todo_show_categories = 0;
 
 int todo_item_dstatus_highlight_overdue = 0;
 int todo_item_dstatus_highlight_due_today = 0;
 int todo_item_dstatus_highlight_not_due_yet = 0;
 
-
 char *todo_overdue_font_text;
 gint todo_current_sort_column = 0;
 gint todo_current_sort_type = GTK_SORT_ASCENDING;
@@ -30,6 +30,8 @@ gboolean todo_style_changed =0;
 gboolean todo_list_autoresize = 1;
 gboolean todo_list_redraw_in_progess = 0;
 static void gncal_todo_init (GncalTodo *todo);
+static void modify_categories ( GtkWidget *widget, GnomeDialog *dialog );
+
 
 
 guint
@@ -63,6 +65,7 @@ ok_button (GtkWidget *widget, GnomeDialog *dialog)
 	GnomeDateEdit *due_date;
 	GtkText *comment;
 	GtkSpinButton *priority;
+	GList* categories;
 
 	ico = gtk_object_get_user_data (GTK_OBJECT (dialog));
 
@@ -71,6 +74,8 @@ ok_button (GtkWidget *widget, GnomeDialog *dialog)
 	due_date = GNOME_DATE_EDIT (gtk_object_get_data(GTK_OBJECT(dialog), "due_date"));
 	priority = GTK_SPIN_BUTTON (gtk_object_get_data(GTK_OBJECT(dialog), "priority"));
 	comment = GTK_TEXT(gtk_object_get_data (GTK_OBJECT(dialog), "comment"));
+	categories = (GList*) gtk_object_get_data (GTK_OBJECT(dialog), "categories");
+
 	if (ico->summary)
 	  g_free (ico->summary);
 	if (ico->comment)
@@ -81,6 +86,13 @@ ok_button (GtkWidget *widget, GnomeDialog *dialog)
 	ico->comment = gtk_editable_get_chars( GTK_EDITABLE(comment), 0, -1);
 	ico->user_data = NULL;
 
+	/* free old category list */
+	if ( ico->categories ) 
+		g_list_free ( ico->categories );
+
+	ico->categories = categories_as_ico_categories ( categories );
+	
+
 	if (ico->new) {
 		gnome_calendar_add_object (todo->calendar, ico);
 		ico->new = FALSE;
@@ -91,6 +103,7 @@ ok_button (GtkWidget *widget, GnomeDialog *dialog)
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
+
 static void
 cancel_button (GtkWidget *widget, GnomeDialog *dialog)
 {
@@ -108,6 +121,7 @@ cancel_button (GtkWidget *widget, GnomeDialog *dialog)
 
 }
 
+
 static gint
 delete_event (GtkWidget *widget, GdkEvent *event, GnomeDialog *dialog)
 {
@@ -115,6 +129,7 @@ delete_event (GtkWidget *widget, GdkEvent *event, GnomeDialog *dialog)
 	return TRUE;
 }
 
+
 static void
 simple_todo_editor (GncalTodo *todo, iCalObject *ico)
 {
@@ -129,13 +144,18 @@ simple_todo_editor (GncalTodo *todo, iCalObject *ico)
 	GtkWidget *comment_internal_box;
 	GtkWidget *comment_sep;
 	GtkWidget *w;
+	GtkWidget *pri_cat_box;
 	GtkWidget *pri_box;
 	GtkWidget *pri_label;
 	GtkWidget *pri_spin;
 	GtkObject *pri_adj;
+	GtkWidget *cat_box;
+	GtkWidget *cat_label;
+	GtkWidget *cat_entry;
+	GtkWidget *cat_button;
 
 	GtkWidget *entry;
-
+	
 	dialog = gnome_dialog_new (ico->new ? _("Create to-do item") : _("Edit to-do item"),
 				   GNOME_STOCK_BUTTON_OK,
 				   GNOME_STOCK_BUTTON_CANCEL,
@@ -146,18 +166,17 @@ simple_todo_editor (GncalTodo *todo, iCalObject *ico)
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show (hbox);
 
-
 	due_box = gtk_hbox_new (FALSE, 4);
 	gtk_container_border_width (GTK_CONTAINER (due_box), 4);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), due_box, FALSE, FALSE, 0);
 	gtk_widget_show (due_box);
 
-	pri_box = gtk_hbox_new (FALSE, 4);
-	gtk_container_border_width (GTK_CONTAINER (pri_box), 4);
-	gtk_box_pack_start(GTK_BOX (GNOME_DIALOG (dialog)->vbox), pri_box, FALSE, FALSE, 0);
-	gtk_widget_show (pri_box);
+	pri_cat_box = gtk_hbox_new (FALSE, 4);
+	gtk_container_border_width (GTK_CONTAINER (pri_cat_box), 4);
+	gtk_box_pack_start(GTK_BOX (GNOME_DIALOG (dialog)->vbox), pri_cat_box, FALSE, FALSE, 0);
+	gtk_widget_show (pri_cat_box);
 	
-	comment_box = gtk_hbox_new (FALSE, 4);
+        comment_box = gtk_hbox_new (FALSE, 4);
 	gtk_container_border_width (GTK_CONTAINER (comment_box), 4);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), comment_box, FALSE, FALSE, 0);
 	gtk_widget_show (comment_box);
@@ -177,7 +196,6 @@ simple_todo_editor (GncalTodo *todo, iCalObject *ico)
 	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 	gtk_widget_show (entry);
 
-
 	due_label = gtk_label_new (_("Due Date:"));
 	gtk_box_pack_start (GTK_BOX (due_box), due_label, FALSE, FALSE, 0);
 	gtk_widget_show (due_label);
@@ -187,9 +205,30 @@ simple_todo_editor (GncalTodo *todo, iCalObject *ico)
 	gtk_box_pack_start (GTK_BOX (due_box), due_entry, TRUE, TRUE, 0);
 	gtk_widget_show (due_entry);
 
+	pri_box   = gtk_hbox_new (FALSE, 4);
 	pri_label = gtk_label_new (_("Priority:"));
 	gtk_box_pack_start (GTK_BOX (pri_box), pri_label, FALSE, FALSE, 0);
 	gtk_widget_show (pri_label);
+	gtk_widget_show (pri_box);
+
+	cat_box    = gtk_hbox_new (FALSE, 4);
+	cat_label  = gtk_label_new (_("Categories:"));
+	cat_entry  = gtk_entry_new ();
+	gtk_entry_set_editable (GTK_ENTRY(cat_entry), FALSE );
+	gtk_entry_set_text (GTK_ENTRY(cat_entry), ico_categories_as_string(ico->categories));
+	cat_button = gtk_button_new_with_label (_("Modify"));
+	gtk_signal_connect (GTK_OBJECT(cat_button), "clicked", modify_categories, dialog );
+	gtk_box_pack_start (GTK_BOX (cat_box), cat_label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (cat_box), cat_entry, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (cat_box), cat_button, FALSE, FALSE, 0);
+	gtk_widget_show ( cat_box );
+	gtk_widget_show ( cat_label );
+	gtk_widget_show ( cat_entry );
+	gtk_widget_show ( cat_button );
+
+
+	gtk_box_pack_start ( GTK_BOX (pri_cat_box), pri_box, FALSE, FALSE, 0);
+	gtk_box_pack_start ( GTK_BOX (pri_cat_box), cat_box, FALSE, FALSE, 0);
 
 	pri_adj = gtk_adjustment_new (5.0, 1.0, 9.0, 1.0, 3.0, 0.0);
 	pri_spin = gtk_spin_button_new (GTK_ADJUSTMENT(pri_adj), 0.0, 0);
@@ -208,9 +247,6 @@ simple_todo_editor (GncalTodo *todo, iCalObject *ico)
 	gtk_box_pack_start (GTK_BOX (comment_internal_box), comment_label, TRUE, TRUE, 0);
 	gtk_widget_show (comment_label);
 
-
-
-
 	comment_text = gtk_text_new (NULL, NULL);
 	gtk_text_set_editable (GTK_TEXT (comment_text), TRUE);
 	gtk_text_set_word_wrap( GTK_TEXT(comment_text), TRUE);
@@ -231,7 +267,9 @@ simple_todo_editor (GncalTodo *todo, iCalObject *ico)
 	gtk_object_set_data (GTK_OBJECT (dialog), "due_date", due_entry);
 	gtk_object_set_data (GTK_OBJECT (dialog), "priority", pri_spin);
 	gtk_object_set_data (GTK_OBJECT (dialog), "comment", comment_text);
-	
+	gtk_object_set_data (GTK_OBJECT (dialog), "cat_entry", cat_entry);
+	gtk_object_set_data (GTK_OBJECT (dialog), "categories", ico_categories_as_categories (ico->categories));
+
 	gnome_dialog_button_connect (GNOME_DIALOG (dialog), 0, (GtkSignalFunc) ok_button, dialog);
 	gnome_dialog_button_connect (GNOME_DIALOG (dialog), 1, (GtkSignalFunc) cancel_button, dialog);
 
@@ -315,6 +353,29 @@ delete_activated (GtkWidget *widget, GncalTodo *todo)
 }
 
 
+/*
+ * modify the categories for this item
+ *
+ */
+static void
+modify_categories ( GtkWidget *widget, GnomeDialog *dialog)
+{
+	GList *categories;
+	GtkEntry *cat_entry;
+	iCalObject *ico = gtk_object_get_user_data (GTK_OBJECT(dialog));
+       
+	categories = ico_categories_as_categories ( ico->categories ); 
+ 
+	modify_cats_dialog ( GTK_WIDGET(dialog), &categories );
+	gtk_object_set_data ( GTK_OBJECT(dialog), "categories", categories );
+	
+	cat_entry = gtk_object_get_data (GTK_OBJECT(dialog), "cat_entry" );
+	gtk_entry_set_text ( cat_entry, categories_as_string ( categories ));
+}
+
+
+
+
 static void
 clist_row_selected (GtkCList *clist, gint row, gint column, GdkEventButton *event, GncalTodo *todo)
 {
@@ -418,19 +479,21 @@ gncal_todo_init (GncalTodo *todo)
 	GtkWidget *w;
 	GtkWidget *sw;
 	GtkWidget *hbox;
-	gchar *titles[4] = {
+	gchar *titles[5] = {
 	    N_("Summary"),
 	    N_("Due Date"),
 	    N_("Priority"),
-	    N_("Time Left")
+	    N_("Time Left"),
+	    N_("Categories")
 	};
-	char *tmp[4];
+	char *tmp[5];
 	tmp[0] = _(titles[0]);
 	tmp[1] = _(titles[1]);
 	tmp[2] = _(titles[2]);
 	tmp[3] = _(titles[3]);
+	tmp[4] = _(titles[4]);
 
-	gtk_box_set_spacing (GTK_BOX (todo), 4);
+	gtk_box_set_spacing (GTK_BOX (todo), 5);
 
 	/* Label */
 
@@ -447,7 +510,7 @@ gncal_todo_init (GncalTodo *todo)
 	gtk_widget_show (sw);
 
 
-	w = gtk_clist_new_with_titles(4, tmp);
+	w = gtk_clist_new_with_titles(5, tmp);
 
 	todo->clist = GTK_CLIST (w);
 	gtk_clist_set_selection_mode (todo->clist, GTK_SELECTION_BROWSE);
@@ -618,7 +681,7 @@ static void
 insert_in_clist (GncalTodo *todo, iCalObject *ico)
 {
 	int i;
-	char *text[4];
+	char *text[5];
 	char time_remaining_buffer[100];
 	time_t time_remain;
 	todo_remaining_time_form time_remaining_form;
@@ -630,8 +693,7 @@ insert_in_clist (GncalTodo *todo, iCalObject *ico)
 	int days = 0;
 	int hours = 0;
 	int minutes = 0;
-	int seconds = 0; 
-	
+	int seconds = 0;
 	
 	/* an array for the styles of items */
 	static GtkStyle *dstatus_styles[TODO_ITEM_DSTATUS_LAST_DUE_STATUS];
@@ -651,8 +713,6 @@ insert_in_clist (GncalTodo *todo, iCalObject *ico)
 	}
 
 	  
-       
-
 
 	text[0] =  ico->summary;
 
@@ -760,6 +820,18 @@ insert_in_clist (GncalTodo *todo, iCalObject *ico)
 	else
 	  text[2] = NULL;
 
+	/*
+	 * categories, comma-separated 
+	 */
+	if ( ico->categories && todo_show_categories ) {
+		
+	        text[4] = ico_categories_as_string ( ico->categories ); /* FIXME: leaking here ?! */ 
+	} else
+
+		text[4] = NULL;
+
+
+
 	i = gtk_clist_append (todo->clist, text);
 	
 	gtk_clist_set_row_data (todo->clist, i, ico);
@@ -842,6 +914,14 @@ gncal_todo_update (GncalTodo *todo, iCalObject *ico, int flags)
 	  gtk_clist_set_column_visibility (todo->clist, 2, 1);
 	else
 	  gtk_clist_set_column_visibility (todo->clist, 2, 0);
+
+	
+	if ( todo_show_categories ) 
+
+		gtk_clist_set_column_visibility ( todo->clist, 4, 1);
+	else
+		gtk_clist_set_column_visibility ( todo->clist, 4, 0);
+
 	
 	/* free the memory locations that were used in the previous display */
 	for (current_list = todo->data_ptrs; current_list != NULL; current_list = g_slist_next(current_list)){
@@ -875,3 +955,10 @@ gncal_todo_update (GncalTodo *todo, iCalObject *ico, int flags)
 
 
 
+
+
+
+
+
+
+
