@@ -51,6 +51,9 @@ int startup_hidden = 0;
 /* If true, enable debug output for alarms */
 int debug_alarms = 0;
 
+/* If true, move calendar to new day at midnight */
+int roll_at_midnight = 0;
+
 /* The array of color properties -- keep in sync with the enumeration defined in main.h.  The color
  * values specified here are the defaults for the program.
  */
@@ -190,6 +193,7 @@ init_calendar (void)
 	day_end    = range_check_hour (gnome_config_get_int  ("/calendar/Calendar/Day end=17"));
 	am_pm_flag = gnome_config_get_bool ("/calendar/Calendar/AM PM flag=0");
 	week_starts_on_monday = gnome_config_get_bool ("/calendar/Calendar/Week starts on Monday=0");
+	roll_at_midnight = gnome_config_get_bool ("/calendar/Calendar/Display new date at midnight=0");
 
 	if (day_end < day_begin){
 		day_begin = 8;
@@ -226,6 +230,9 @@ init_calendar (void)
 	
 	todo_show_priority = gnome_config_get_bool("/calendar/Todo/show_priority");
 
+	todo_show_categories = gnome_config_get_bool("/calendar/Todo/show_categories");
+
+
 	/* read alarm settings */
 	beep_on_display = gnome_config_get_bool ("/calendar/alarms/beep_on_display=FALSE");
 	enable_aalarm_timeout = gnome_config_get_bool ("/calendar/alarms/enable_audio_timeout=FALSE");
@@ -284,7 +291,7 @@ display_objedit (GtkWidget *widget, GnomeCalendar *gcal)
 	/* Default to the day the user is looking at */
 	ico = ical_new ("", user_name, "");
 	ico->new = 1;
-	ico->dtstart = time_add_minutes (gcal->current_display, day_begin * 60);
+	ico->dtstart = time_add_minutes (gnome_calendar_current_display (gcal), day_begin * 60);
 	ico->dtend   = time_add_minutes (ico->dtstart, day_begin * 60 + 30 );
 
 	ee = event_editor_new (gcal, ico);
@@ -470,7 +477,6 @@ open_calendar_cmd (GtkWidget *widget, void *data)
 				   GTK_OBJECT (fs));
 
 	gtk_widget_show (GTK_WIDGET (fs));
-	gtk_grab_add (GTK_WIDGET (fs)); /* Yes, it is modal, so sue me */
 }
 
 static void
@@ -480,18 +486,20 @@ save_ok (GtkWidget *widget, GtkFileSelection *fs)
 	gchar *fname;
 
 	gcal = GNOME_CALENDAR (gtk_object_get_user_data (GTK_OBJECT (fs)));
+	
+	/* FIXME: this is causing a GTK-Warning */
 	gtk_window_set_wmclass (GTK_WINDOW (gcal), "gnomecal", "gnomecal");
 
 	fname = g_strdup (gtk_file_selection_get_filename (fs));
 	calendar_save (gcal->cal, fname);
 	g_free(fname);
-	gtk_main_quit ();
+	gtk_widget_destroy (fs);
 }
 
 static gint
 close_save (GtkWidget *w)
 {
-	gtk_main_quit ();
+	gtk_widget_destroy (w);
 	return TRUE;
 }
 
@@ -513,9 +521,6 @@ save_as_calendar_cmd (GtkWidget *widget, void *data)
 				   GTK_SIGNAL_FUNC (close_save),
 				   GTK_OBJECT (fs));
 	gtk_widget_show (GTK_WIDGET (fs));
-	gtk_grab_add (GTK_WIDGET (fs)); /* Yes, it is modal, so sue me even more */
-	gtk_main ();
-	gtk_widget_destroy (GTK_WIDGET (fs));
 }
 
 static void
