@@ -53,6 +53,7 @@ static GtkWidget *todo_item_highlight_not_due_yet;
 static GtkWidget *todo_item_highlight_due_today;
 
 static GtkWidget *priority_show_button;
+static GtkWidget *categories_show_button;
 
 /* Widgets for the alarm page */
 static GtkWidget *enable_display_beep;
@@ -148,8 +149,8 @@ prop_apply_todo(void)
 	todo_item_dstatus_highlight_due_today = GTK_TOGGLE_BUTTON(todo_item_highlight_due_today)->active;
 
 	todo_show_priority = GTK_TOGGLE_BUTTON (priority_show_button)->active;
-
 	todo_show_time_remaining = GTK_TOGGLE_BUTTON (todo_item_time_remaining_show_button)->active;
+	todo_show_categories = GTK_TOGGLE_BUTTON (categories_show_button)->active;
 
 	/* storing the values */
 
@@ -161,6 +162,8 @@ prop_apply_todo(void)
         gnome_config_set_bool("/calendar/Todo/highlight_not_due_yet", todo_item_dstatus_highlight_not_due_yet);
 	gnome_config_set_bool("/calendar/Todo/show_due_date", todo_show_due_date);
 	gnome_config_set_bool("/calendar/Todo/show_priority", todo_show_priority);
+	gnome_config_set_bool("/calendar/Todo/show_categories", todo_show_categories);
+
 	/* need to sync our config changes. */
 	gnome_config_sync ();
 
@@ -604,6 +607,7 @@ build_list_options_frame(void)
 	due_date_show_button = gtk_check_button_new_with_label (_("Due Date"));
 	priority_show_button = gtk_check_button_new_with_label (_("Priority"));
 	todo_item_time_remaining_show_button = gtk_check_button_new_with_label (_("Time Until Due"));
+	categories_show_button = gtk_check_button_new_with_label (_("Categories"));
 
 	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(due_date_show_button), todo_show_due_date);
 	gtk_signal_connect (GTK_OBJECT(due_date_show_button),
@@ -626,6 +630,14 @@ build_list_options_frame(void)
 			    (GtkSignalFunc) todo_option_set,
 			    NULL);
 	gtk_box_pack_start (GTK_BOX (vbox), priority_show_button, FALSE, FALSE, 0);
+
+	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(categories_show_button), todo_show_categories);
+	gtk_signal_connect (GTK_OBJECT(categories_show_button), 
+			    "clicked",
+			    (GtkSignalFunc) todo_option_set,
+			    NULL);
+	gtk_box_pack_start (GTK_BOX (vbox), categories_show_button, FALSE, FALSE, 0);
+
 	return frame;
 }
 static GtkWidget *
