@@ -197,6 +197,13 @@ static void gnomecard_prop_apply(GtkWidget *widget, int page)
 	/* link to new phone list */
 	crd->phone.l = phonelist_get_entries (ce->phonelist);
 
+        /* Update revision */
+        tt = time(NULL);
+        tm = gmtime(&tt);
+        crd->rev.utc = 0;
+        crd->rev.tm = *tm;
+        crd->rev.prop.used = TRUE;
+        
 	/* key data */
         MY_FREE(crd->key.data);
 	
